(*
Copyright  2001-2002 Franois Thomasset, all rights reserved.
Copying is covered by the GNU General Public License (GPL).
 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

$Revision: 1.1.1.1 $
$Date: 2002/03/08 08:31:57 $
$Author: thomasse $
 
*)
open Maple;;

(* I found the following useful for debugging the abstract syntax tree.
   They produce strings displaying the intermediate representation for
   various Maple constructions.
 *)
let kelop = function
  | PlusOp -> " + " | MinusOp -> " - " | MultOp -> " * " | DivOp -> " div"
  | ExpoOp -> " ^ "
  | OrOp -> " or " | AndOp -> " and "
  | LTOp -> " < " | GTOp -> " < " | LEOp -> " <= " | GEOp -> " >= " | NEOp -> " <> " | EQOp -> " = "
  | ModOp -> " mod "
  | Union -> " union " | Minus -> " minus " | Intersect -> " intersect "
  | NeutralMulOp -> " &* "
  | NeutralOp (_) -> " &? ";;

let kel_unop = function
  | UnaryMinus -> " UnaryMinus " | UnaryPlus -> " UnaryPlus " | NotOp -> " not " ;;

let rec kelnom = function
  | NameString (Ident(n)) -> "NameString(Ident(" ^ n ^ "))"
  | NameString (QuotedString(n)) -> "NameString(QuotedString(" ^ n ^ "))"
  | NameIndexed (n,es) -> "NameIndexed(" ^ kelnom(n) ^ "," ^ (kelexps es) ^ ")"
  | NameConcat (n,N_int(i)) -> "NameConcat(" ^ kelnom(n) ^ "," ^ (string_of_int i) ^ ")"
  | NameConcat (n,N_str(s)) -> "NameConcat(" ^ kelnom(n) ^ ",\"" ^ s ^ "\")"
  | NameConcat (n,N_expr(e)) -> "NameConcat(" ^ kelnom(n) ^ "," ^ (kelexp e) ^ ")"
and kelnoms = function
  | [] -> ""
  | [n] -> kelnom n
  | n :: reste -> (kelnom n) ^ "," ^ (kelnoms reste)
and kelexp = function
  | ArrowExpr (ns,e) -> "ArrowExpr([" ^ kelnoms(ns) ^ "] -> " ^ kelexp(e) ^ ")"
  | BinExpr ( { left = l ; bin_op = oper; right = r } ) ->
      "BinExpr(" ^ kelexp(l) ^ kelop(oper) ^ kelexp(r) ^ ")"
  | Factorial (e) -> "Factorial(" ^ kelexp(e) ^ ")"
  | UnaryExpr (op,e) -> "UnaryExpr(" ^ kel_unop(op) ^ "," ^ kelexp(e) ^ ")"
  | SeqExpr (Some e,ee) -> "SeqExpr(" ^ kelexp(e) ^ "," ^ kelexp(ee) ^ ")"
  | SeqExpr (None,ee) -> "SeqExpr(" ^ kelexp(ee) ^ ")"
  | IntervalExpr (e1,e2) -> "IntervalExpr(" ^ kelexp(e1) ^ "," ^ kelexp(e2) ^ ")"
  | NameExpr (n) -> kelnom (n)
  | StringExpr (s) -> "StringExpr(\"" ^ s ^ "\")"
  | IntExpr (i) -> "IntExpr(" ^ (string_of_int i) ^ ")"
  | FloatExpr (f1,f2) -> "FloatExpr(" ^ (string_of_int f1) ^ "," ^ (string_of_int f2) ^ ")"
  | CallExpr ( { callee = n; args = a } ) -> "CallExpr(" ^ kelnom(n) ^ "(" ^ (kelexps a) ^ "))"
  | FunctionalCallExpr ( { callee_f = n; fargs = a } )
    ->
      "FunctionalCallExpr(" ^ kelnom(n)
      ^ "("
      ^ (kelexpss a)
      ^ "))"
  | UnevaluatedExpr (e) -> kelexp (e)
  | ListExpr (es) -> "ListExpr(" ^ (kelexps es) ^ ")"
  | SetExpr (es) -> "SetExpr(" ^ (kelexps es) ^ ")"
  | ProcDef (_) -> "ProcDef()"
  | ParenExpr ([e]) -> "ParenExpr(" ^ kelexp(e) ^ ")"
  | ParenExpr (es) -> "ParenExpr(" ^ (kelexps es) ^ ")"
  | ComposeExpr (f,g) -> "ComposeExpr(" ^ (kelexp f) ^ "," ^ (kelexp g) ^ ")"
  | RepeatComposeExpr (f,g) -> "RepeatComposeExpr" ^ (kelexp f) ^ "," ^ (kelexp g) ^ ""
and kelexps = function
  | [] -> ""
  | [e] -> kelexp e
  | e :: reste -> (kelexp e) ^ "," ^ (kelexps reste)
and kelexpss = function
  | [] -> ""
  | [ee] -> "[" ^ (kelexps ee) ^ "]"
  | ee :: reste
    -> (kelexps ee) ^ "," ^ (kelexpss reste)
and kelstat = function
  | AssignStat ({ lhs_of_assign = ns ; rhs = es }) ->
      "AssignStat(" ^ (kelnoms ns) ^ "=" ^ (kelexps es) ^ ")"
  | ExprStat(e) -> "ExprStat(" ^ (kelexp e) ^ ")"
  | ReadStat(e) -> "ReadStat(" ^ (kelexp e) ^ ")"
  | SaveStat(ns,Ident(s)) -> "SaveStat(" ^ (kelnoms ns) ^ ",Ident(" ^ s ^ "))"
  | SaveStat(ns,QuotedString(s)) -> "SaveStat(" ^ (kelnoms ns) ^ ",QuotedString(" ^ s ^ "))"
  | IfThenElseStat ({cond = c; then_part = t; alternatives = alts; else_part = e})
      ->
	"IfThenElseStat(" ^ (kelexp c)
	^ ", Then:" ^ (kelstats t)
	^ ", Alts: " ^ (kelalts alts)
	^ ", Else : " ^ (kelstats e)
	^ ")"
  | ForStat ({ loop_index = i ;
	       from_expr = from_e ; by_expr = by_e ; to_expr = to_e ; while_expr = wh_e ;
	       body = b
	     })
    ->
      let loop_s = match i with | Some n -> (kelnom n) | _ -> ""
      in
      let from_s = match from_e with | Some e -> " From " ^ (kelexp e) | _ -> ""
      in
      let by_s = match by_e with | Some e -> " To " ^ (kelexp e) | _ -> ""
      in
      let wh_s = match wh_e with | Some e -> " While " ^ (kelexp e) | _ -> ""
      in
      "For " ^ loop_s ^ from_s ^ by_s ^ wh_s ^ " Do " ^ (kelstats b) ^ " Done "
  | ForInStat ( { loop_var = n; in_expr = in_e; in_while = wh_e; in_body = b } )
      ->
	let wh_s = match wh_e with | Some e -> " While " ^ (kelexp e) | _ -> ""
	in
	"For " ^ (kelnom n) ^ " In " ^ (kelexp in_e) ^ wh_s ^ " Do " ^ (kelstats b) ^ " Done "
  | StopStat -> "StopStat"
  | EmptyStat -> "EmptyStat"
and kelstats = function
  | [] -> ""
  | { comments = _; sep = separator; stat = s } :: reste
    ->
      (kelsep separator) ^ " " ^ (kelstat s) ^ (kelstats reste)
and kelsep = function
  | SemiColon -> ";" | Colon -> ":" | NoSep -> " "
and kelalts = function
  | [] -> ""
  | [{ cond_of_alt = c; stats_of_alt = sts }] ->
      "ElseIf (" ^ (kelexp c) ^ " Then " ^ (kelstats sts)
  | { cond_of_alt = c; stats_of_alt = sts } :: reste ->
      "ElseIf (" ^ (kelexp c) ^ " Then " ^ (kelstats sts)
      ^ "," ^ (kelalts reste);;
